@@ -17,7 +17,6 @@ from threading import Timer
 from typing import (
     IO,
     Callable,
-    Dict,
     Iterable,
     List,
     Match,
@@ -32,27 +31,26 @@ from typing import (
 
 import psutil
 import shellescape
+from prov.model import PROV
 from schema_salad.sourceline import SourceLine
 from schema_salad.utils import json_dump, json_dumps
 from typing_extensions import TYPE_CHECKING
 
-from prov.model import PROV
-
 from .builder import Builder, HasReqsHints
-from .context import RuntimeContext, getdefault
+from .context import RuntimeContext
 from .errors import UnsupportedRequirement, WorkflowException
 from .loghandler import _logger
 from .pathmapper import MapperEnt, PathMapper
 from .process import stage_files
 from .secrets import SecretStore
 from .utils import (
-    DEFAULT_TMP_PREFIX,
     CWLObjectType,
     CWLOutputType,
     DirectoryType,
     OutputCallbackType,
     bytes2str_in_dicts,
     copytree_with_merge,
+    create_tmp_dir,
     ensure_non_writable,
     ensure_writable,
     onWindows,
@@ -381,11 +379,7 @@ class JobBase(HasReqsHints, metaclass=ABCMeta):
                 stderr_path=stderr_path,
                 env=env,
                 cwd=self.outdir,
-                job_dir=tempfile.mkdtemp(
-                    prefix=getdefault(
-                        runtimeContext.tmp_outdir_prefix, DEFAULT_TMP_PREFIX
-                    )
-                ),
+                make_job_dir=lambda: runtimeContext.create_outdir(),
                 job_script_contents=job_script_contents,
                 timelimit=self.timelimit,
                 name=self.name,
@@ -415,7 +409,7 @@ class JobBase(HasReqsHints, metaclass=ABCMeta):
                     )
                 else:
                     raise ValueError(
-                        "'lsiting' in self.generatefiles but no "
+                        "'listing' in self.generatefiles but no "
                         "generatemapper was setup."
                     )
 
@@ -601,14 +595,16 @@ class ContainerCommandLineJob(JobBase, metaclass=ABCMeta):
         self,
         r: CWLObjectType,
         pull_image: bool,
-        force_pull: bool = False,
-        tmp_outdir_prefix: str = DEFAULT_TMP_PREFIX,
+        force_pull: bool,
+        tmp_outdir_prefix: str,
     ) -> Optional[str]:
         pass
 
     @abstractmethod
     def create_runtime(
-        self, env: MutableMapping[str, str], runtime_context: RuntimeContext,
+        self,
+        env: MutableMapping[str, str],
+        runtime_context: RuntimeContext,
     ) -> Tuple[List[str], Optional[str]]:
         """Return the list of commands to run the selected container engine."""
 
@@ -655,9 +651,8 @@ class ContainerCommandLineJob(JobBase, metaclass=ABCMeta):
     ) -> str:
         """Create the file and add a mapping."""
         if not host_outdir_tgt:
-            tmp_dir, tmp_prefix = os.path.split(tmpdir_prefix)
             new_file = os.path.join(
-                tempfile.mkdtemp(prefix=tmp_prefix, dir=tmp_dir),
+                create_tmp_dir(tmpdir_prefix),
                 os.path.basename(volume.target),
             )
         writable = True if volume.type == "CreateWritableFile" else False
@@ -769,10 +764,8 @@ class ContainerCommandLineJob(JobBase, metaclass=ABCMeta):
                         self.get_from_requirements(
                             docker_req,
                             runtimeContext.pull_image,
-                            getdefault(runtimeContext.force_docker_pull, False),
-                            getdefault(
-                                runtimeContext.tmp_outdir_prefix, DEFAULT_TMP_PREFIX
-                            ),
+                            runtimeContext.force_docker_pull,
+                            runtimeContext.tmp_outdir_prefix,
                         )
                     )
                 if img_id is None:
@@ -916,7 +909,7 @@ def _job_popen(
     stderr_path: Optional[str],
     env: MutableMapping[str, str],
     cwd: str,
-    job_dir: str,
+    make_job_dir: Callable[[], str],
     job_script_contents: Optional[str] = None,
     timelimit: Optional[int] = None,
     name: Optional[str] = None,
@@ -965,8 +958,7 @@ def _job_popen(
             def terminate():  # type: () -> None
                 try:
                     _logger.warning(
-                        "[job %s] exceeded time limit of %d seconds and will"
-                        "be terminated",
+                        "[job %s] exceeded time limit of %d seconds and will be terminated",
                         name,
                         timelimit,
                     )
@@ -1012,11 +1004,12 @@ def _job_popen(
             "stdin_path": stdin_path,
         }
 
-        with open(
-            os.path.join(job_dir, "job.json"), mode="w", encoding="utf-8"
-        ) as job_file:
-            json_dump(job_description, job_file, ensure_ascii=False)
+        job_dir = make_job_dir()
         try:
+            with open(
+                os.path.join(job_dir, "job.json"), mode="w", encoding="utf-8"
+            ) as job_file:
+                json_dump(job_description, job_file, ensure_ascii=False)
             job_script = os.path.join(job_dir, "run_job.bash")
             with open(job_script, "wb") as _:
                 _.write(job_script_contents.encode("utf-8"))
