@@ -8,7 +8,6 @@ import urllib
 import uuid
 from typing import (
     Any,
-    Callable,
     Dict,
     List,
     MutableMapping,
@@ -19,14 +18,18 @@ from typing import (
     cast,
 )
 
-import requests.sessions
-
 from ruamel.yaml.comments import CommentedMap, CommentedSeq
-from schema_salad import schema
-from schema_salad.ref_resolver import ContextType, Fetcher, Loader, SubLoader, file_uri
+from schema_salad.exceptions import ValidationException
+from schema_salad.ref_resolver import Loader, file_uri
+from schema_salad.schema import validate_doc
 from schema_salad.sourceline import SourceLine, cmap
-from schema_salad.utils import json_dumps
-from schema_salad.validate import ValidationException
+from schema_salad.utils import (
+    ContextType,
+    FetcherCallableType,
+    IdxResultType,
+    ResolveType,
+    json_dumps,
+)
 
 from . import process, update
 from .context import LoadingContext
@@ -34,7 +37,7 @@ from .errors import WorkflowException
 from .loghandler import _logger
 from .process import Process, get_schema, shortname
 from .update import ALLUPDATES
-from .utils import visit_class
+from .utils import CWLObjectType, ResolverType, visit_class
 
 jobloaderctx = {
     "cwl": "https://w3id.org/cwl/cwl#",
@@ -59,14 +62,11 @@ overrides_ctx = {
 }  # type: ContextType
 
 
-FetcherConstructorType = Callable[
-    [Dict[str, Union[str, bool]], requests.sessions.Session], Fetcher
-]
-ResolverType = Callable[[Loader, Union[str, Dict[str, Any]]], str]
-
-
-def default_loader(fetcher_constructor=None, enable_dev=False, doc_cache=True):
-    # type: (Optional[FetcherConstructorType], bool, bool) -> Loader
+def default_loader(
+    fetcher_constructor: Optional[FetcherCallableType] = None,
+    enable_dev: bool = False,
+    doc_cache: bool = True,
+) -> Loader:
     return Loader(
         jobloaderctx,
         fetcher_constructor=fetcher_constructor,
@@ -78,7 +78,7 @@ def default_loader(fetcher_constructor=None, enable_dev=False, doc_cache=True):
 def resolve_tool_uri(
     argsworkflow: str,
     resolver: Optional[ResolverType] = None,
-    fetcher_constructor: Optional[FetcherConstructorType] = None,
+    fetcher_constructor: Optional[FetcherCallableType] = None,
     document_loader: Optional[Loader] = None,
 ) -> Tuple[str, str]:
 
@@ -90,9 +90,9 @@ def resolve_tool_uri(
     elif os.path.exists(os.path.abspath(argsworkflow)):
         uri = file_uri(str(os.path.abspath(argsworkflow)))
     elif resolver is not None:
-        if document_loader is None:
-            document_loader = default_loader(fetcher_constructor)
-        uri = resolver(document_loader, argsworkflow)
+        uri = resolver(
+            document_loader or default_loader(fetcher_constructor), argsworkflow
+        )
 
     if uri is None:
         raise ValidationException("Not found: '%s'" % argsworkflow)
@@ -105,7 +105,7 @@ def resolve_tool_uri(
 
 
 def fetch_document(
-    argsworkflow: Union[str, Dict[str, Any]],
+    argsworkflow: Union[str, CWLObjectType],
     loadingContext: Optional[LoadingContext] = None,
 ) -> Tuple[LoadingContext, CommentedMap, str]:
     """Retrieve a CWL document."""
@@ -127,19 +127,27 @@ def fetch_document(
             resolver=loadingContext.resolver,
             document_loader=loadingContext.loader,
         )
-        workflowobj = loadingContext.loader.fetch(fileuri)
+        workflowobj = cast(CommentedMap, loadingContext.loader.fetch(fileuri))
         return loadingContext, workflowobj, uri
-    if isinstance(argsworkflow, dict):
-        uri = argsworkflow["id"] if argsworkflow.get("id") else "_:" + str(uuid.uuid4())
-        workflowobj = cast(CommentedMap, cmap(argsworkflow, fn=uri))
+    if isinstance(argsworkflow, MutableMapping):
+        uri = (
+            cast(str, argsworkflow["id"])
+            if argsworkflow.get("id")
+            else "_:" + str(uuid.uuid4())
+        )
+        workflowobj = cast(
+            CommentedMap, cmap(cast(Dict[str, Any], argsworkflow), fn=uri)
+        )
         loadingContext.loader.idx[uri] = workflowobj
         return loadingContext, workflowobj, uri
     raise ValidationException("Must be URI or object: '%s'" % argsworkflow)
 
 
-def _convert_stdstreams_to_files(workflowobj):
-    # type: (Union[Dict[str, Any], List[Dict[str, Any]]]) -> None
-
+def _convert_stdstreams_to_files(
+    workflowobj: Union[
+        CWLObjectType, MutableSequence[Union[CWLObjectType, str, int]], str
+    ]
+) -> None:
     if isinstance(workflowobj, MutableMapping):
         if workflowobj.get("class") == "CommandLineTool":
             with SourceLine(
@@ -151,10 +159,12 @@ def _convert_stdstreams_to_files(workflowobj):
                 outputs = workflowobj.get("outputs", [])
                 if not isinstance(outputs, CommentedSeq):
                     raise ValidationException('"outputs" section is not ' "valid.")
-                for out in workflowobj.get("outputs", []):
+                for out in cast(
+                    MutableSequence[CWLObjectType], workflowobj.get("outputs", [])
+                ):
                     if not isinstance(out, CommentedMap):
                         raise ValidationException(
-                            "Output '{}' is not a valid " "OutputParameter.".format(out)
+                            "Output '{}' is not a valid OutputParameter.".format(out)
                         )
                     for streamtype in ["stdout", "stderr"]:
                         if out.get("type") == streamtype:
@@ -176,7 +186,9 @@ def _convert_stdstreams_to_files(workflowobj):
                                 workflowobj[streamtype] = filename
                             out["type"] = "File"
                             out["outputBinding"] = cmap({"glob": filename})
-            for inp in workflowobj.get("inputs", []):
+            for inp in cast(
+                MutableSequence[CWLObjectType], workflowobj.get("inputs", [])
+            ):
                 if inp.get("type") == "stdin":
                     if "inputBinding" in inp:
                         raise ValidationException(
@@ -190,20 +202,39 @@ def _convert_stdstreams_to_files(workflowobj):
                         )
                     else:
                         workflowobj["stdin"] = (
-                            "$(inputs.%s.path)" % inp["id"].rpartition("#")[2]
+                            "$(inputs.%s.path)"
+                            % cast(str, inp["id"]).rpartition("#")[2]
                         )
                         inp["type"] = "File"
         else:
             for entry in workflowobj.values():
-                _convert_stdstreams_to_files(entry)
+                _convert_stdstreams_to_files(
+                    cast(
+                        Union[
+                            CWLObjectType,
+                            MutableSequence[Union[CWLObjectType, str, int]],
+                            str,
+                        ],
+                        entry,
+                    )
+                )
     if isinstance(workflowobj, MutableSequence):
         for entry in workflowobj:
-            _convert_stdstreams_to_files(entry)
+            _convert_stdstreams_to_files(
+                cast(
+                    Union[
+                        CWLObjectType,
+                        MutableSequence[Union[CWLObjectType, str, int]],
+                        str,
+                    ],
+                    entry,
+                )
+            )
 
 
-def _add_blank_ids(workflowobj):
-    # type: (Union[Dict[str, Any], List[Dict[str, Any]]]) -> None
-
+def _add_blank_ids(
+    workflowobj: Union[CWLObjectType, MutableSequence[Union[CWLObjectType, str]]]
+) -> None:
     if isinstance(workflowobj, MutableMapping):
         if (
             "run" in workflowobj
@@ -213,20 +244,29 @@ def _add_blank_ids(workflowobj):
         ):
             workflowobj["run"]["id"] = str(uuid.uuid4())
         for entry in workflowobj.values():
-            _add_blank_ids(entry)
+            _add_blank_ids(
+                cast(
+                    Union[CWLObjectType, MutableSequence[Union[CWLObjectType, str]]],
+                    entry,
+                )
+            )
     if isinstance(workflowobj, MutableSequence):
         for entry in workflowobj:
-            _add_blank_ids(entry)
+            _add_blank_ids(
+                cast(
+                    Union[CWLObjectType, MutableSequence[Union[CWLObjectType, str]]],
+                    entry,
+                )
+            )
 
 
 def resolve_and_validate_document(
-    loadingContext,  # type: LoadingContext
-    workflowobj,  # type: Union[CommentedMap, CommentedSeq]
-    uri,  # type: str
-    preprocess_only=False,  # type: bool
-    skip_schemas=None,  # type: Optional[bool]
-):
-    # type: (...) -> Tuple[LoadingContext, str]
+    loadingContext: LoadingContext,
+    workflowobj: Union[CommentedMap, CommentedSeq],
+    uri: str,
+    preprocess_only: bool = False,
+    skip_schemas: Optional[bool] = None,
+) -> Tuple[LoadingContext, str]:
     """Validate a CWL document."""
     if not loadingContext.loader:
         raise ValueError("loadingContext must have a loader.")
@@ -257,8 +297,8 @@ def resolve_and_validate_document(
     if not cwlVersion and fileuri != uri:
         # The tool we're loading is a fragment of a bigger file.  Get
         # the document root element and look for cwlVersion there.
-        metadata = fetch_document(fileuri, loadingContext)[1]  # type: Dict[str, Any]
-        cwlVersion = metadata.get("cwlVersion")
+        metadata = cast(CWLObjectType, fetch_document(fileuri, loadingContext)[1])
+        cwlVersion = cast(str, metadata.get("cwlVersion"))
     if not cwlVersion:
         raise ValidationException(
             "No cwlVersion found. "
@@ -271,7 +311,7 @@ def resolve_and_validate_document(
     if not isinstance(cwlVersion, str):
         with SourceLine(workflowobj, "cwlVersion", ValidationException):
             raise ValidationException(
-                "'cwlVersion' must be a string, " "got {}".format(type(cwlVersion))
+                "'cwlVersion' must be a string, got {}".format(type(cwlVersion))
             )
     # strip out version
     cwlVersion = re.sub(r"^(?:cwl:|https://w3id.org/cwl/cwl#)", "", cwlVersion)
@@ -319,7 +359,7 @@ def resolve_and_validate_document(
     if isinstance(avsc_names, Exception):
         raise avsc_names
 
-    processobj = None  # type: Union[CommentedMap, CommentedSeq, str, None]
+    processobj = None  # type: Optional[ResolveType]
     document_loader = Loader(
         sch_document_loader.ctx,
         schemagraph=sch_document_loader.graph,
@@ -335,10 +375,15 @@ def resolve_and_validate_document(
 
     document_loader.resolve_all(workflowobj, fileuri)
     processobj, metadata = document_loader.resolve_ref(uri)
-    if loadingContext.metadata:
-        metadata = loadingContext.metadata
     if not isinstance(processobj, (CommentedMap, CommentedSeq)):
         raise ValidationException("Workflow must be a CommentedMap or CommentedSeq.")
+
+    if not hasattr(processobj.lc, "filename"):
+        processobj.lc.filename = fileuri
+
+    if loadingContext.metadata:
+        metadata = loadingContext.metadata
+
     if not isinstance(metadata, CommentedMap):
         raise ValidationException(
             "metadata must be a CommentedMap, was %s" % type(metadata)
@@ -360,9 +405,7 @@ def resolve_and_validate_document(
         return loadingContext, uri
 
     if loadingContext.do_validate:
-        schema.validate_doc(
-            avsc_names, processobj, document_loader, loadingContext.strict
-        )
+        validate_doc(avsc_names, processobj, document_loader, loadingContext.strict)
 
     # None means default behavior (do update)
     if loadingContext.do_update in (True, None):
@@ -423,9 +466,9 @@ def make_tool(
 
 
 def load_tool(
-    argsworkflow,  # type: Union[str, Dict[str, Any]]
-    loadingContext=None,  # type: Optional[LoadingContext]
-):  # type: (...) -> Process
+    argsworkflow: Union[str, CWLObjectType],
+    loadingContext: Optional[LoadingContext] = None,
+) -> Process:
 
     loadingContext, workflowobj, uri = fetch_document(argsworkflow, loadingContext)
 
@@ -437,19 +480,38 @@ def load_tool(
 
 
 def resolve_overrides(
-    ov,  # type: CommentedMap
-    ov_uri,  # type: str
-    baseurl,  # type: str
-):  # type: (...) -> List[Dict[str, Any]]
+    ov: IdxResultType,
+    ov_uri: str,
+    baseurl: str,
+) -> List[CWLObjectType]:
     ovloader = Loader(overrides_ctx)
     ret, _ = ovloader.resolve_all(ov, baseurl)
     if not isinstance(ret, CommentedMap):
         raise Exception("Expected CommentedMap, got %s" % type(ret))
     cwl_docloader = get_schema("v1.0")[0]
     cwl_docloader.resolve_all(ret, ov_uri)
-    return cast(List[Dict[str, Any]], ret["http://commonwl.org/cwltool#overrides"])
+    return cast(List[CWLObjectType], ret["http://commonwl.org/cwltool#overrides"])
 
 
-def load_overrides(ov, base_url):  # type: (str, str) -> List[Dict[str, Any]]
+def load_overrides(ov: str, base_url: str) -> List[CWLObjectType]:
     ovloader = Loader(overrides_ctx)
     return resolve_overrides(ovloader.fetch(ov), ov, base_url)
+
+
+def recursive_resolve_and_validate_document(
+    loadingContext: LoadingContext,
+    workflowobj: Union[CommentedMap, CommentedSeq],
+    uri: str,
+    preprocess_only: bool = False,
+    skip_schemas: Optional[bool] = None,
+) -> Tuple[LoadingContext, str, Process]:
+    """Validate a CWL document, checking that a tool object can be built."""
+    loadingContext, uri = resolve_and_validate_document(
+        loadingContext,
+        workflowobj,
+        uri,
+        preprocess_only=preprocess_only,
+        skip_schemas=skip_schemas,
+    )
+    tool = make_tool(uri, loadingContext)
+    return loadingContext, uri, tool
