@@ -5,7 +5,6 @@ import os.path
 import re
 import shutil
 import sys
-import tempfile
 from distutils import spawn
 from subprocess import (  # nosec
     DEVNULL,
@@ -27,6 +26,7 @@ from .loghandler import _logger
 from .pathmapper import MapperEnt, PathMapper
 from .utils import (
     CWLObjectType,
+    create_tmp_dir,
     docker_windows_path_adjust,
     ensure_non_writable,
     ensure_writable,
@@ -260,8 +260,8 @@ class SingularityCommandLineJob(ContainerCommandLineJob):
         self,
         r: CWLObjectType,
         pull_image: bool,
-        force_pull: bool = False,
-        tmp_outdir_prefix: Optional[str] = None,
+        force_pull: bool,
+        tmp_outdir_prefix: str,
     ) -> Optional[str]:
         """
         Return the filename of the Singularity image.
@@ -273,7 +273,7 @@ class SingularityCommandLineJob(ContainerCommandLineJob):
 
         if not self.get_image(cast(Dict[str, str], r), pull_image, force_pull):
             raise WorkflowException(
-                "Container image {} not " "found".format(r["dockerImageId"])
+                "Container image {} not found".format(r["dockerImageId"])
             )
 
         return os.path.abspath(cast(str, r["dockerImageId"]))
@@ -331,9 +331,8 @@ class SingularityCommandLineJob(ContainerCommandLineJob):
             self.append_volume(runtime, volume.resolved, volume.target, writable=True)
             ensure_writable(volume.resolved)
         else:
-            tmp_dir, tmp_prefix = os.path.split(tmpdir_prefix)
             file_copy = os.path.join(
-                tempfile.mkdtemp(prefix=tmp_prefix, dir=tmp_dir),
+                create_tmp_dir(tmpdir_prefix),
                 os.path.basename(volume.resolved),
             )
             shutil.copy(volume.resolved, file_copy)
@@ -352,9 +351,8 @@ class SingularityCommandLineJob(ContainerCommandLineJob):
             if host_outdir_tgt is not None:
                 new_dir = host_outdir_tgt
             else:
-                tmp_dir, tmp_prefix = os.path.split(tmpdir_prefix)
                 new_dir = os.path.join(
-                    tempfile.mkdtemp(prefix=tmp_prefix, dir=tmp_dir),
+                    create_tmp_dir(tmpdir_prefix),
                     os.path.basename(volume.resolved),
                 )
             os.makedirs(new_dir)
@@ -368,9 +366,8 @@ class SingularityCommandLineJob(ContainerCommandLineJob):
                 ensure_writable(host_outdir_tgt)
             else:
                 if not self.inplace_update:
-                    tmp_dir, tmp_prefix = os.path.split(tmpdir_prefix)
                     dir_copy = os.path.join(
-                        tempfile.mkdtemp(prefix=tmp_prefix, dir=tmp_dir),
+                        create_tmp_dir(tmpdir_prefix),
                         os.path.basename(volume.resolved),
                     )
                     shutil.copytree(volume.resolved, dir_copy)
