@@ -2,11 +2,9 @@ import copy
 import datetime
 import functools
 import logging
-import tempfile
 import threading
 from typing import (
     Dict,
-    Generator,
     List,
     MutableMapping,
     MutableSequence,
@@ -29,10 +27,8 @@ from .loghandler import _logger
 from .process import shortname, uniquename
 from .stdfsaccess import StdFsAccess
 from .utils import (
-    DEFAULT_TMP_PREFIX,
     CWLObjectType,
     CWLOutputType,
-    DestinationsType,
     JobsGeneratorType,
     OutputCallbackType,
     ParametersType,
@@ -488,13 +484,7 @@ class WorkflowJob(object):
         self.processStatus = ""
         self.did_callback = False
         self.made_progress = None  # type: Optional[bool]
-
-        if runtimeContext.outdir is not None:
-            self.outdir = runtimeContext.outdir
-        else:
-            self.outdir = tempfile.mkdtemp(
-                prefix=getdefault(runtimeContext.tmp_outdir_prefix, DEFAULT_TMP_PREFIX)
-            )
+        self.outdir = runtimeContext.get_outdir()
 
         self.name = uniquename(
             "workflow {}".format(
