@@ -7,9 +7,10 @@ import sys
 from io import StringIO
 from pathlib import Path
 from typing import Any, Dict, List, Union, cast
+from urllib.parse import urlparse
 
-import py.path
-import pytest  # type: ignore
+import pydot  # type: ignore
+import pytest
 from ruamel.yaml.comments import CommentedMap, CommentedSeq
 from schema_salad.exceptions import ValidationException
 
@@ -27,18 +28,12 @@ from cwltool.process import CWL_IANA
 from cwltool.sandboxjs import JavascriptException
 from cwltool.utils import CWLObjectType, dedup, onWindows
 
-import pydot
-from urllib.parse import urlparse
-
 from .util import (
     get_data,
     get_main_output,
     get_windows_safe_factory,
     needs_docker,
-    needs_singularity,
-    temp_dir,
     windows_needs_docker,
-    working_directory,
 )
 
 sys.argv = [""]
@@ -71,7 +66,7 @@ expression_match = [
 ]
 
 
-@pytest.mark.parametrize("expression,expected", expression_match)  # type: ignore
+@pytest.mark.parametrize("expression,expected", expression_match)
 def test_expression_match(expression: str, expected: bool) -> None:
     match = expr.param_re.match(expression)
     assert (match is not None) == expected
@@ -128,11 +123,105 @@ interpolate_parameters = [
 ]
 
 
-@pytest.mark.parametrize("pattern,expected", interpolate_parameters)  # type: ignore
+@pytest.mark.parametrize("pattern,expected", interpolate_parameters)
 def test_expression_interpolate(pattern: str, expected: Any) -> None:
     assert expr.interpolate(pattern, interpolate_input) == expected
 
 
+parameter_to_expressions = [
+    (
+        "-$(foo)",
+        r"""-{"bar":{"baz":"zab1"},"b ar":{"baz":2},"b'ar":{"baz":true},"b\"ar":{"baz":null}}""",
+    ),
+    ("-$(foo.bar)", """-{"baz":"zab1"}"""),
+    ("-$(foo['bar'])", """-{"baz":"zab1"}"""),
+    ('-$(foo["bar"])', """-{"baz":"zab1"}"""),
+    ("-$(foo.bar.baz)", "-zab1"),
+    ("-$(foo['bar'].baz)", "-zab1"),
+    ("-$(foo['bar'][\"baz\"])", "-zab1"),
+    ("-$(foo.bar['baz'])", "-zab1"),
+    ("-$(foo['b ar'].baz)", "-2"),
+    ("-$(foo['b\\'ar'].baz)", "-true"),
+    ('-$(foo["b\\\'ar"].baz)', "-true"),
+    ("-$(foo['b\\\"ar'].baz)", "-null"),
+    ("$(foo.bar) $(foo.bar)", """{"baz":"zab1"} {"baz":"zab1"}"""),
+    ("$(foo['bar']) $(foo['bar'])", """{"baz":"zab1"} {"baz":"zab1"}"""),
+    ('$(foo["bar"]) $(foo["bar"])', """{"baz":"zab1"} {"baz":"zab1"}"""),
+    ("$(foo.bar.baz) $(foo.bar.baz)", "zab1 zab1"),
+    ("$(foo['bar'].baz) $(foo['bar'].baz)", "zab1 zab1"),
+    ("$(foo['bar'][\"baz\"]) $(foo['bar'][\"baz\"])", "zab1 zab1"),
+    ("$(foo.bar['baz']) $(foo.bar['baz'])", "zab1 zab1"),
+    ("$(foo['b ar'].baz) $(foo['b ar'].baz)", "2 2"),
+    ("$(foo['b\\'ar'].baz) $(foo['b\\'ar'].baz)", "true true"),
+    ('$(foo["b\\\'ar"].baz) $(foo["b\\\'ar"].baz)', "true true"),
+    ("$(foo['b\\\"ar'].baz) $(foo['b\\\"ar'].baz)", "null null"),
+]
+
+
+@pytest.mark.parametrize("pattern,expected", parameter_to_expressions)
+def test_parameter_to_expression(pattern: str, expected: Any) -> None:
+    """Test the interpolate convert_to_expression feature."""
+    expression = expr.interpolate(pattern, {}, convert_to_expression=True)
+    assert isinstance(expression, str)
+    assert (
+        expr.interpolate(
+            expression,
+            {},
+            jslib=expr.jshead([], interpolate_input),
+            fullJS=True,
+            debug=True,
+        )
+        == expected
+    )
+
+
+param_to_expr_interpolate_escapebehavior = (
+    ("\\$(foo.bar.baz)", "$(foo.bar.baz)", 1),
+    ("\\\\$(foo.bar.baz)", "\\zab1", 1),
+    ("\\\\\\$(foo.bar.baz)", "\\$(foo.bar.baz)", 1),
+    ("\\\\\\\\$(foo.bar.baz)", "\\\\zab1", 1),
+    ("\\$foo", "$foo", 1),
+    ("\\foo", "foo", 1),
+    ("\\x", "x", 1),
+    ("\\\\x", "\\x", 1),
+    ("\\\\\\x", "\\x", 1),
+    ("\\\\\\\\x", "\\\\x", 1),
+    ("\\$(foo.bar.baz)", "$(foo.bar.baz)", 2),
+    ("\\\\$(foo.bar.baz)", "\\zab1", 2),
+    ("\\\\\\$(foo.bar.baz)", "\\$(foo.bar.baz)", 2),
+    ("\\\\\\\\$(foo.bar.baz)", "\\\\zab1", 2),
+    ("\\$foo", "\\$foo", 2),
+    ("\\foo", "\\foo", 2),
+    ("\\x", "\\x", 2),
+    ("\\\\x", "\\x", 2),
+    ("\\\\\\x", "\\\\x", 2),
+    ("\\\\\\\\x", "\\\\x", 2),
+)
+
+
+@pytest.mark.parametrize(
+    "pattern,expected,behavior", param_to_expr_interpolate_escapebehavior
+)
+def test_parameter_to_expression_interpolate_escapebehavior(
+    pattern: str, expected: str, behavior: int
+) -> None:
+    """Test escaping behavior in an convert_to_expression context."""
+    expression = expr.interpolate(
+        pattern, {}, escaping_behavior=behavior, convert_to_expression=True
+    )
+    assert isinstance(expression, str)
+    assert (
+        expr.interpolate(
+            expression,
+            {},
+            jslib=expr.jshead([], interpolate_input),
+            fullJS=True,
+            debug=True,
+        )
+        == expected
+    )
+
+
 interpolate_bad_parameters = [
     ("$(fooz)"),
     ("$(foo.barz)"),
@@ -163,7 +252,7 @@ interpolate_bad_parameters = [
 ]
 
 
-@pytest.mark.parametrize("pattern", interpolate_bad_parameters)  # type: ignore
+@pytest.mark.parametrize("pattern", interpolate_bad_parameters)
 def test_expression_interpolate_failures(pattern: str) -> None:
     result = None
     try:
@@ -197,7 +286,7 @@ interpolate_escapebehavior = (
 )
 
 
-@pytest.mark.parametrize("pattern,expected,behavior", interpolate_escapebehavior)  # type: ignore
+@pytest.mark.parametrize("pattern,expected,behavior", interpolate_escapebehavior)
 def test_expression_interpolate_escapebehavior(
     pattern: str, expected: str, behavior: int
 ) -> None:
@@ -208,7 +297,7 @@ def test_expression_interpolate_escapebehavior(
     )
 
 
-@windows_needs_docker  # type: ignore
+@windows_needs_docker
 def test_factory() -> None:
     factory = get_windows_safe_factory()
     echo = factory.make(get_data("tests/echo.cwl"))
@@ -248,13 +337,14 @@ def test_factory_partial_scatter() -> None:
     with pytest.raises(cwltool.factory.WorkflowStatus) as err_info:
         factory.make(get_data("tests/wf/scatterfail.cwl"))()
 
-    err = err_info.value
+    result = err_info.value.out
+    assert isinstance(result, dict)
     assert (
-        err.out["out"][0]["checksum"] == "sha1$e5fa44f2b31c1fb553b6021e7360d07d5d91ff5e"
+        result["out"][0]["checksum"] == "sha1$e5fa44f2b31c1fb553b6021e7360d07d5d91ff5e"
     )
-    assert err.out["out"][1] is None
+    assert result["out"][1] is None
     assert (
-        err.out["out"][2]["checksum"] == "sha1$a3db5c13ff90a36963278c6a39e4ee3c22e2a436"
+        result["out"][2]["checksum"] == "sha1$a3db5c13ff90a36963278c6a39e4ee3c22e2a436"
     )
 
 
@@ -266,15 +356,14 @@ def test_factory_partial_output() -> None:
     with pytest.raises(cwltool.factory.WorkflowStatus) as err_info:
         factory.make(get_data("tests/wf/wffail.cwl"))()
 
-    err = err_info.value
-    assert (
-        err.out["out1"]["checksum"] == "sha1$e5fa44f2b31c1fb553b6021e7360d07d5d91ff5e"
-    )
-    assert err.out["out2"] is None
+    result = err_info.value.out
+    assert isinstance(result, dict)
+    assert result["out1"]["checksum"] == "sha1$e5fa44f2b31c1fb553b6021e7360d07d5d91ff5e"
+    assert result["out2"] is None
 
 
 def test_scandeps() -> None:
-    obj = {
+    obj: CWLObjectType = {
         "id": "file:///example/foo.cwl",
         "steps": [
             {
@@ -337,15 +426,18 @@ def test_scandeps() -> None:
             return p
         raise Exception("test case can't load things")
 
-    scanned_deps = cwltool.process.scandeps(
-        cast(str, obj["id"]),
-        obj,
-        {"$import", "run"},
-        {"$include", "$schemas", "location"},
-        loadref,
+    scanned_deps = cast(
+        List[Dict[str, Any]],
+        cwltool.process.scandeps(
+            cast(str, obj["id"]),
+            obj,
+            {"$import", "run"},
+            {"$include", "$schemas", "location"},
+            loadref,
+        ),
     )
 
-    scanned_deps.sort(key=lambda k: k["basename"])
+    scanned_deps.sort(key=lambda k: cast(str, k["basename"]))
 
     expected_deps = [
         {
@@ -397,11 +489,20 @@ def test_scandeps() -> None:
 
     assert scanned_deps == expected_deps
 
-    scanned_deps = cwltool.process.scandeps(
-        cast(str, obj["id"]), obj, set(("run"),), set(), loadref
+    scanned_deps2 = cast(
+        List[Dict[str, Any]],
+        cwltool.process.scandeps(
+            cast(str, obj["id"]),
+            obj,
+            set(
+                ("run"),
+            ),
+            set(),
+            loadref,
+        ),
     )
 
-    scanned_deps.sort(key=lambda k: k["basename"])
+    scanned_deps2.sort(key=lambda k: cast(str, k["basename"]))
 
     expected_deps = [
         {
@@ -414,7 +515,7 @@ def test_scandeps() -> None:
         }
     ]
 
-    assert scanned_deps == expected_deps
+    assert scanned_deps2 == expected_deps
 
 
 def test_trick_scandeps() -> None:
@@ -587,7 +688,7 @@ source_to_sink = [
 ]
 
 
-@pytest.mark.parametrize("name, source, sink, expected", source_to_sink)  # type: ignore
+@pytest.mark.parametrize("name, source, sink, expected", source_to_sink)
 def test_compare_types(
     name: str, source: Dict[str, Any], sink: Dict[str, Any], expected: bool
 ) -> None:
@@ -613,7 +714,7 @@ source_to_sink_strict = [
 ]
 
 
-@pytest.mark.parametrize("name, source, sink, expected", source_to_sink_strict)  # type: ignore
+@pytest.mark.parametrize("name, source, sink, expected", source_to_sink_strict)
 def test_compare_types_strict(
     name: str, source: Dict[str, Any], sink: Dict[str, Any], expected: bool
 ) -> None:
@@ -742,7 +843,7 @@ typechecks = [
 ]
 
 
-@pytest.mark.parametrize(  # type: ignore
+@pytest.mark.parametrize(
     "src_type,sink_type,link_merge,value_from,expected_type", typechecks
 )
 def test_typechecking(
@@ -851,7 +952,8 @@ def test_print_dot() -> None:
     # print Workflow
     cwl_path = get_data("tests/wf/revsort.cwl")
     cwl_posix_path = Path(cwl_path).as_posix()
-    expected_dot = pydot.graph_from_dot_data("""
+    expected_dot = pydot.graph_from_dot_data(
+        """
     digraph {{
         graph [bgcolor="#eeeeee",
                 clusterrank=local,
@@ -891,14 +993,25 @@ def test_print_dot() -> None:
         "file://{cwl_posix_path}#workflow_input" -> "file://{cwl_posix_path}#rev";
         "file://{cwl_posix_path}#reverse_sort" -> "file://{cwl_posix_path}#sorted";
 }}
-    """.format(cwl_posix_path=cwl_posix_path))[0]
+    """.format(
+            cwl_posix_path=cwl_posix_path
+        )
+    )[0]
     stdout = StringIO()
     assert main(["--print-dot", cwl_path], stdout=stdout) == 0
     computed_dot = pydot.graph_from_dot_data(stdout.getvalue())[0]
     computed_edges = sorted(
-        [(urlparse(source).fragment, urlparse(target).fragment) for source, target in computed_dot.obj_dict['edges']])
+        [
+            (urlparse(source).fragment, urlparse(target).fragment)
+            for source, target in computed_dot.obj_dict["edges"]
+        ]
+    )
     expected_edges = sorted(
-        [(urlparse(source).fragment, urlparse(target).fragment) for source, target in expected_dot.obj_dict['edges']])
+        [
+            (urlparse(source).fragment, urlparse(target).fragment)
+            for source, target in expected_dot.obj_dict["edges"]
+        ]
+    )
     assert computed_edges == expected_edges
 
     # print CommandLineTool
@@ -910,7 +1023,7 @@ def test_print_dot() -> None:
 test_factors = [(""), ("--parallel"), ("--debug"), ("--parallel --debug")]
 
 
-@pytest.mark.parametrize("factor", test_factors)  # type: ignore
+@pytest.mark.parametrize("factor", test_factors)
 def test_js_console_cmd_line_tool(factor: str) -> None:
     for test_file in ("js_output.cwl", "js_output_workflow.cwl"):
         commands = factor.split()
@@ -925,7 +1038,7 @@ def test_js_console_cmd_line_tool(factor: str) -> None:
         assert error_code == 0, stderr
 
 
-@pytest.mark.parametrize("factor", test_factors)  # type: ignore
+@pytest.mark.parametrize("factor", test_factors)
 def test_no_js_console(factor: str) -> None:
     for test_file in ("js_output.cwl", "js_output_workflow.cwl"):
         commands = factor.split()
@@ -936,64 +1049,66 @@ def test_no_js_console(factor: str) -> None:
         assert "[err] Error message" not in stderr
 
 
-@needs_docker  # type: ignore
-@pytest.mark.parametrize("factor", test_factors)  # type: ignore
-def test_cid_file_dir(tmpdir: py.path.local, factor: str) -> None:
+@needs_docker
+@pytest.mark.parametrize("factor", test_factors)
+def test_cid_file_dir(tmp_path: Path, factor: str) -> None:
+    """Test --cidfile-dir option works."""
     test_file = "cache_test_workflow.cwl"
-    cwd = tmpdir.chdir()
+    cwd = Path.cwd()
+    os.chdir(tmp_path)
     commands = factor.split()
-    commands.extend(["--cidfile-dir", str(tmpdir), get_data("tests/wf/" + test_file)])
+    commands.extend(["--cidfile-dir", str(tmp_path), get_data("tests/wf/" + test_file)])
     error_code, stdout, stderr = get_main_output(commands)
     assert "completed success" in stderr
     assert error_code == 0
-    cidfiles_count = sum(1 for _ in tmpdir.visit(fil="*"))
+    cidfiles_count = sum(1 for _ in tmp_path.glob("**/*"))
     assert cidfiles_count == 2
-    cwd.chdir()
-    tmpdir.remove(ignore_errors=True)
+    os.chdir(cwd)
 
 
-@needs_docker  # type: ignore
-@pytest.mark.parametrize("factor", test_factors)  # type: ignore
-def test_cid_file_dir_arg_is_file_instead_of_dir(
-    tmpdir: py.path.local, factor: str
-) -> None:
+@needs_docker
+@pytest.mark.parametrize("factor", test_factors)
+def test_cid_file_dir_arg_is_file_instead_of_dir(tmp_path: Path, factor: str) -> None:
+    """Test --cidfile-dir with a file produces the correct error."""
     test_file = "cache_test_workflow.cwl"
-    bad_cidfile_dir = str(tmpdir.ensure("cidfile-dir-actually-a-file"))
+    bad_cidfile_dir = tmp_path / "cidfile-dir-actually-a-file"
+    bad_cidfile_dir.touch()
     commands = factor.split()
     commands.extend(
-        ["--cidfile-dir", bad_cidfile_dir, get_data("tests/wf/" + test_file)]
+        ["--cidfile-dir", str(bad_cidfile_dir), get_data("tests/wf/" + test_file)]
     )
     error_code, _, stderr = get_main_output(commands)
     assert "is not a directory, please check it first" in stderr, stderr
     assert error_code == 2 or error_code == 1, stderr
-    tmpdir.remove(ignore_errors=True)
 
 
-@needs_docker  # type: ignore
-@pytest.mark.parametrize("factor", test_factors)  # type: ignore
-def test_cid_file_non_existing_dir(tmpdir: py.path.local, factor: str) -> None:
+@needs_docker
+@pytest.mark.parametrize("factor", test_factors)
+def test_cid_file_non_existing_dir(tmp_path: Path, factor: str) -> None:
+    """Test that --cachedir with a bad path should produce a specific error."""
     test_file = "cache_test_workflow.cwl"
-    bad_cidfile_dir = str(tmpdir.join("cidfile-dir-badpath"))
+    bad_cidfile_dir = tmp_path / "cidfile-dir-badpath"
     commands = factor.split()
     commands.extend(
         [
             "--record-container-id",
             "--cidfile-dir",
-            bad_cidfile_dir,
+            str(bad_cidfile_dir),
             get_data("tests/wf/" + test_file),
         ]
     )
     error_code, _, stderr = get_main_output(commands)
     assert "directory doesn't exist, please create it first" in stderr, stderr
     assert error_code == 2 or error_code == 1, stderr
-    tmpdir.remove(ignore_errors=True)
 
 
-@needs_docker  # type: ignore
-@pytest.mark.parametrize("factor", test_factors)  # type: ignore
-def test_cid_file_w_prefix(tmpdir: py.path.local, factor: str) -> None:
+@needs_docker
+@pytest.mark.parametrize("factor", test_factors)
+def test_cid_file_w_prefix(tmp_path: Path, factor: str) -> None:
+    """Test that --cidfile-prefix works."""
     test_file = "cache_test_workflow.cwl"
-    cwd = tmpdir.chdir()
+    cwd = Path.cwd()
+    os.chdir(tmp_path)
     try:
         commands = factor.split()
         commands.extend(
@@ -1005,17 +1120,16 @@ def test_cid_file_w_prefix(tmpdir: py.path.local, factor: str) -> None:
         )
         error_code, stdout, stderr = get_main_output(commands)
     finally:
-        listing = tmpdir.listdir()
-        cwd.chdir()
-        cidfiles_count = sum(1 for _ in tmpdir.visit(fil="pytestcid*"))
-        tmpdir.remove(ignore_errors=True)
+        listing = tmp_path.iterdir()
+        os.chdir(cwd)
+        cidfiles_count = sum(1 for _ in tmp_path.glob("**/pytestcid*"))
     assert "completed success" in stderr
     assert error_code == 0
-    assert cidfiles_count == 2, "{}/n{}".format(listing, stderr)
+    assert cidfiles_count == 2, "{}/n{}".format(list(listing), stderr)
 
 
-@needs_docker  # type: ignore
-@pytest.mark.parametrize("factor", test_factors)  # type: ignore
+@needs_docker
+@pytest.mark.parametrize("factor", test_factors)
 def test_secondary_files_v1_1(factor: str) -> None:
     test_file = "secondary-files.cwl"
     test_job_file = "secondary-files-job.yml"
@@ -1038,8 +1152,8 @@ def test_secondary_files_v1_1(factor: str) -> None:
     assert error_code == 0
 
 
-@needs_docker  # type: ignore
-@pytest.mark.parametrize("factor", test_factors)  # type: ignore
+@needs_docker
+@pytest.mark.parametrize("factor", test_factors)
 def test_secondary_files_v1_0(factor: str) -> None:
     test_file = "secondary-files-string-v1.cwl"
     test_job_file = "secondary-files-job.yml"
@@ -1061,50 +1175,52 @@ def test_secondary_files_v1_0(factor: str) -> None:
     assert error_code == 0
 
 
-@needs_docker  # type: ignore
-@pytest.mark.parametrize("factor", test_factors)  # type: ignore
-def test_wf_without_container(tmpdir: py.path.local, factor: str) -> None:
+@needs_docker
+@pytest.mark.parametrize("factor", test_factors)
+def test_wf_without_container(tmp_path: Path, factor: str) -> None:
+    """Confirm that we can run a workflow without a container."""
     test_file = "hello-workflow.cwl"
-    with temp_dir("cwltool_cache") as cache_dir:
-        commands = factor.split()
-        commands.extend(
-            [
-                "--cachedir",
-                cache_dir,
-                "--outdir",
-                str(tmpdir),
-                get_data("tests/wf/" + test_file),
-                "--usermessage",
-                "hello",
-            ]
-        )
-        error_code, _, stderr = get_main_output(commands)
+    cache_dir = str(tmp_path / "cwltool_cache")
+    commands = factor.split()
+    commands.extend(
+        [
+            "--cachedir",
+            cache_dir,
+            "--outdir",
+            str(tmp_path / "outdir"),
+            get_data("tests/wf/" + test_file),
+            "--usermessage",
+            "hello",
+        ]
+    )
+    error_code, _, stderr = get_main_output(commands)
 
     assert "completed success" in stderr
     assert error_code == 0
 
 
-@needs_docker  # type: ignore
-@pytest.mark.parametrize("factor", test_factors)  # type: ignore
-def test_issue_740_fixed(factor: str) -> None:
+@needs_docker
+@pytest.mark.parametrize("factor", test_factors)
+def test_issue_740_fixed(tmp_path: Path, factor: str) -> None:
+    """Confirm that re-running a particular workflow with caching suceeds."""
     test_file = "cache_test_workflow.cwl"
-    with temp_dir("cwltool_cache") as cache_dir:
-        commands = factor.split()
-        commands.extend(["--cachedir", cache_dir, get_data("tests/wf/" + test_file)])
-        error_code, _, stderr = get_main_output(commands)
+    cache_dir = str(tmp_path / "cwltool_cache")
+    commands = factor.split()
+    commands.extend(["--cachedir", cache_dir, get_data("tests/wf/" + test_file)])
+    error_code, _, stderr = get_main_output(commands)
 
-        assert "completed success" in stderr
-        assert error_code == 0
+    assert "completed success" in stderr
+    assert error_code == 0
 
-        commands = factor.split()
-        commands.extend(["--cachedir", cache_dir, get_data("tests/wf/" + test_file)])
-        error_code, _, stderr = get_main_output(commands)
+    commands = factor.split()
+    commands.extend(["--cachedir", cache_dir, get_data("tests/wf/" + test_file)])
+    error_code, _, stderr = get_main_output(commands)
 
-        assert "Output of job will be cached in" not in stderr
-        assert error_code == 0, stderr
+    assert "Output of job will be cached in" not in stderr
+    assert error_code == 0, stderr
 
 
-@needs_docker  # type: ignore
+@needs_docker
 def test_compute_checksum() -> None:
     runtime_context = RuntimeContext()
     runtime_context.compute_checksum = True
@@ -1121,9 +1237,9 @@ def test_compute_checksum() -> None:
     assert result["checksum"] == "sha1$327fc7aedf4f6b69a42a7c8b808dc5a7aff61376"
 
 
-@needs_docker  # type: ignore
-@pytest.mark.parametrize("factor", test_factors)  # type: ignore
-def test_no_compute_chcksum(tmpdir: py.path.local, factor: str) -> None:
+@needs_docker
+@pytest.mark.parametrize("factor", test_factors)
+def test_no_compute_chcksum(tmp_path: Path, factor: str) -> None:
     test_file = "tests/wf/wc-tool.cwl"
     job_file = "tests/wf/wc-job.json"
     commands = factor.split()
@@ -1131,7 +1247,7 @@ def test_no_compute_chcksum(tmpdir: py.path.local, factor: str) -> None:
         [
             "--no-compute-checksum",
             "--outdir",
-            str(tmpdir),
+            str(tmp_path),
             get_data(test_file),
             get_data(job_file),
         ]
@@ -1142,8 +1258,8 @@ def test_no_compute_chcksum(tmpdir: py.path.local, factor: str) -> None:
     assert "checksum" not in stdout
 
 
-@pytest.mark.skipif(onWindows(), reason="udocker is Linux/macOS only")  # type: ignore
-@pytest.mark.parametrize("factor", test_factors)  # type: ignore
+@pytest.mark.skipif(onWindows(), reason="udocker is Linux/macOS only")
+@pytest.mark.parametrize("factor", test_factors)
 def test_bad_userspace_runtime(factor: str) -> None:
     test_file = "tests/wf/wc-tool.cwl"
     job_file = "tests/wf/wc-job.json"
@@ -1161,8 +1277,8 @@ def test_bad_userspace_runtime(factor: str) -> None:
     assert error_code == 1
 
 
-@windows_needs_docker  # type: ignore
-@pytest.mark.parametrize("factor", test_factors)  # type: ignore
+@windows_needs_docker
+@pytest.mark.parametrize("factor", test_factors)
 def test_bad_basecommand(factor: str) -> None:
     test_file = "tests/wf/missing-tool.cwl"
     commands = factor.split()
@@ -1172,8 +1288,8 @@ def test_bad_basecommand(factor: str) -> None:
     assert error_code == 1
 
 
-@needs_docker  # type: ignore
-@pytest.mark.parametrize("factor", test_factors)  # type: ignore
+@needs_docker
+@pytest.mark.parametrize("factor", test_factors)
 def test_bad_basecommand_docker(factor: str) -> None:
     test_file = "tests/wf/missing-tool.cwl"
     commands = factor.split()
@@ -1183,7 +1299,7 @@ def test_bad_basecommand_docker(factor: str) -> None:
     assert error_code == 1
 
 
-@pytest.mark.parametrize("factor", test_factors)  # type: ignore
+@pytest.mark.parametrize("factor", test_factors)
 def test_v1_0_position_expression(factor: str) -> None:
     test_file = "tests/echo-position-expr.cwl"
     test_job = "tests/echo-position-expr-job.yml"
@@ -1194,8 +1310,8 @@ def test_v1_0_position_expression(factor: str) -> None:
     assert error_code == 1
 
 
-@windows_needs_docker  # type: ignore
-@pytest.mark.parametrize("factor", test_factors)  # type: ignore
+@windows_needs_docker
+@pytest.mark.parametrize("factor", test_factors)
 def test_optional_numeric_output_0(factor: str) -> None:
     test_file = "tests/wf/optional-numerical-output-0.cwl"
     commands = factor.split()
@@ -1207,8 +1323,8 @@ def test_optional_numeric_output_0(factor: str) -> None:
     assert json.loads(stdout)["out"] == 0
 
 
-@pytest.mark.parametrize("factor", test_factors)  # type: ignore
-@windows_needs_docker  # type: ignore
+@pytest.mark.parametrize("factor", test_factors)
+@windows_needs_docker
 def test_env_filtering(factor: str) -> None:
     test_file = "tests/env.cwl"
     commands = factor.split()
@@ -1233,6 +1349,7 @@ def test_env_filtering(factor: str) -> None:
         env=None,
     )
     sh_name_b, sh_name_err = process.communicate()
+    assert sh_name_b
     sh_name = sh_name_b.decode("utf-8").strip()
 
     assert "completed success" in stderr, (error_code, stdout, stderr)
@@ -1253,7 +1370,7 @@ def test_env_filtering(factor: str) -> None:
     assert result == target, (error_code, sh_name, sh_name_err, details, stdout, stderr)
 
 
-@windows_needs_docker  # type: ignore
+@windows_needs_docker
 def test_v1_0_arg_empty_prefix_separate_false() -> None:
     test_file = "tests/arg-empty-prefix-separate-false.cwl"
     error_code, stdout, stderr = get_main_output(
@@ -1261,3 +1378,25 @@ def test_v1_0_arg_empty_prefix_separate_false() -> None:
     )
     assert "completed success" in stderr
     assert error_code == 0
+
+
+def test_scatter_output_filenames(tmp_path: Path) -> None:
+    """If a scatter step produces identically named output then confirm that the final output is renamed correctly."""
+    cwd = Path.cwd()
+    os.chdir(tmp_path)
+    rtc = RuntimeContext()
+    rtc.outdir = str(cwd)
+    factory = cwltool.factory.Factory(runtime_context=rtc)
+    output_names = ["output.txt", "output.txt_2", "output.txt_3"]
+    scatter_workflow = factory.make(get_data("tests/scatter_numbers.cwl"))
+    result = scatter_workflow(range=3)
+    assert isinstance(result, dict)
+    assert "output" in result
+
+    locations = sorted([element["location"] for element in result["output"]])
+
+    assert (
+        locations[0].endswith("output.txt")
+        and locations[1].endswith("output.txt_2")
+        and locations[2].endswith("output.txt_3")
+    ), "Locations {} do not end with {}".format(locations, output_names)
