@@ -1,25 +1,26 @@
-import pytest  # type: ignore
-
-import sys
+"""Tests of the experimental MPI extension."""
+import json
 import os.path
+import sys
 from io import StringIO
+from pathlib import Path
+from typing import Any, Generator, List, MutableMapping, Optional, Tuple
+
+import pkg_resources
+import pytest
 from ruamel import yaml
 from ruamel.yaml.comments import CommentedMap, CommentedSeq
-import json
-from pathlib import Path
-from typing import Any, Optional, List, MutableMapping, Generator, Tuple, cast
-import pkg_resources
 from schema_salad.avro.schema import Names
 
-from .util import get_data, working_directory, windows_needs_docker
-
+import cwltool.load_tool
+import cwltool.singularity
+import cwltool.udocker
 from cwltool.command_line_tool import CommandLineTool
 from cwltool.context import LoadingContext, RuntimeContext
 from cwltool.main import main
 from cwltool.mpi import MpiConfig, MPIRequirementName
-import cwltool.singularity
-import cwltool.udocker
-import cwltool.load_tool
+
+from .util import get_data, windows_needs_docker, working_directory
 
 
 def test_mpi_conf_defaults() -> None:
@@ -35,10 +36,10 @@ def test_mpi_conf_defaults() -> None:
 
 def test_mpi_conf_unknownkeys() -> None:
     with pytest.raises(TypeError):
-        MpiConfig(runner="mpiexec", foo="bar")
+        MpiConfig(runner="mpiexec", foo="bar")  # type: ignore
 
 
-@pytest.fixture(scope="class")  # type: ignore
+@pytest.fixture(scope="class")
 def fake_mpi_conf(tmp_path_factory: Any) -> Generator[str, None, None]:
     """
     Make a super simple mpirun-alike for applications that don't actually use MPI.
@@ -133,7 +134,7 @@ class TestMpiRun:
         assert conf_obj.default_nproc == 1
         assert conf_obj.extra_flags == ["--no-fail"]
 
-    @windows_needs_docker  # type: ignore
+    @windows_needs_docker
     def test_simple_mpi_tool(self, fake_mpi_conf: str, tmp_path: Path) -> None:
         stdout = StringIO()
         stderr = StringIO()
@@ -152,7 +153,7 @@ class TestMpiRun:
                 pids = [int(line) for line in pidfile]
             assert len(pids) == 2
 
-    @windows_needs_docker  # type: ignore
+    @windows_needs_docker
     def test_simple_mpi_nproc_expr(self, fake_mpi_conf: str, tmp_path: Path) -> None:
         np = 4
         input_file = make_processes_input(np, tmp_path)
@@ -173,7 +174,7 @@ class TestMpiRun:
                 pids = [int(line) for line in pidfile]
             assert len(pids) == np
 
-    @windows_needs_docker  # type: ignore
+    @windows_needs_docker
     def test_mpi_workflow(self, fake_mpi_conf: str, tmp_path: Path) -> None:
         np = 3
         input_file = make_processes_input(np, tmp_path)
@@ -194,7 +195,7 @@ class TestMpiRun:
                 lc = int(lc_file.read())
                 assert lc == np
 
-    @windows_needs_docker  # type: ignore
+    @windows_needs_docker
     def test_environment(
         self, fake_mpi_conf: str, tmp_path: Path, monkeypatch: Any
     ) -> None:
@@ -222,7 +223,8 @@ class TestMpiRun:
 
 def test_env_passing(monkeypatch: Any) -> None:
     config = MpiConfig(
-        env_pass=["A", "B", "LONG_NAME"], env_pass_regex=["TOOLNAME", "MPI_.*_CONF"],
+        env_pass=["A", "B", "LONG_NAME"],
+        env_pass_regex=["TOOLNAME", "MPI_.*_CONF"],
     )
 
     env = {}  # type: MutableMapping[str, str]
@@ -280,12 +282,14 @@ def test_env_passing(monkeypatch: Any) -> None:
 
 
 # Reading the schema is super slow - cache for the session
-@pytest.fixture(scope="session")  # type: ignore
+@pytest.fixture(scope="session")
 def schema_ext11() -> Generator[Names, None, None]:
     with pkg_resources.resource_stream("cwltool", "extensions-v1.1.yml") as res:
         ext11 = res.read().decode("utf-8")
         cwltool.process.use_custom_schema("v1.1", "http://commonwl.org/cwltool", ext11)
-        yield cwltool.process.get_schema("v1.1")[1]
+        schema = cwltool.process.get_schema("v1.1")[1]
+        assert isinstance(schema, Names)
+        yield schema
 
 
 mpiReq = CommentedMap({"class": MPIRequirementName, "processes": 1})
